package com.fjhx.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.thread.ThreadUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fjhx.config.TaskPoolConfig;
import com.fjhx.constants.SendConstants;
import com.fjhx.entity.*;
import com.fjhx.service.*;
import com.fjhx.utils.EmailEngineUtil;
import com.fjhx.utils.RetryUtil;
import com.fjhx.vo.BindingVo;
import com.fjhx.vo.EmailMailboxVo;
import com.fjhx.vo.MessageDetailVo;
import com.fjhx.vo.MessageVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executor;
import java.util.stream.Collectors;

@Service
public class AccountServiceImpl implements IAccountService {

    @Autowired
    private IEmailInfoService emailInfoService;

    @Autowired
    private IEmailMailboxService emailMailboxService;

    @Autowired
    private IEmailMessageService emailMessageService;

    @Autowired
    private IEmailMessageSendService emailMessageSendService;

    @Autowired
    private IEmailMessageAttachmentService emailMessageAttachmentService;

    @Qualifier(TaskPoolConfig.taskExecutor)
    @Autowired
    private Executor executor;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public EmailInfo binding(BindingVo bindingVo) {

        String email = bindingVo.getEmail();

        EmailInfo emailInfo = emailInfoService.getOne(Wrappers.<EmailInfo>lambdaQuery().eq(EmailInfo::getEmail, email));
        // 如果存在，直接返回邮箱信息
        if (emailInfo != null) {
            return emailInfo;
        }

        // 添加账号
        EmailEngineUtil.createAccount(bindingVo);
        // 保存账号数据
        emailInfo = saveEmailInfo(bindingVo);
        // 查询邮箱文件夹
        List<EmailMailboxVo.MailboxesDTO> mailboxList = EmailEngineUtil.getMailboxList(email);
        // 添加邮箱文件夹
        List<EmailMailbox> emailMailboxList = saveEmailMailbox(mailboxList, emailInfo.getId(), email);
        // 异步遍历文件夹下的所有邮件
        asyncReadEmail(email, emailMailboxList);
        return emailInfo;
    }

    /**
     * 添加邮件信息
     */
    private EmailInfo saveEmailInfo(BindingVo bindingVo) {
        EmailInfo emailInfo = BeanUtil.copyProperties(bindingVo, EmailInfo.class);
        emailInfoService.save(emailInfo);
        return emailInfo;
    }

    /**
     * 添加邮箱文件夹
     */
    private List<EmailMailbox> saveEmailMailbox(List<EmailMailboxVo.MailboxesDTO> mailboxDTOList, Long emailInfoId, String email) {

        List<EmailMailbox> mailboxList = mailboxDTOList.stream().map(item -> {
            EmailMailbox emailMailbox = BeanUtil.copyProperties(item, EmailMailbox.class);
            emailMailbox.setEmailInfoId(emailInfoId);
            emailMailbox.setEmail(email);
            emailMailbox.setIsShow(true);
            emailMailbox.setInitialize(false);
            return emailMailbox;
        }).collect(Collectors.toList());

        emailMailboxService.saveBatch(mailboxList);

        return mailboxList;
    }

    /**
     * 异步读取文件
     */
    private void asyncReadEmail(String email, List<EmailMailbox> emailMailboxList) {
        executor.execute(() -> {
            ThreadUtil.sleep(5000);

            // 记录上次消息id,防止循环查询分页过程中接收到了新邮件，出现邮件重复问题
            List<String> lastMessageId = new ArrayList<>();

            for (EmailMailbox emailMailbox : emailMailboxList) {
                EmailEngineUtil.handleMessageList(
                        email,
                        emailMailbox.getPath(),
                        messageList -> {
                            List<EmailMessage> emailMessageList = new ArrayList<>();
                            List<EmailMessageSend> emailMessageSendList = new ArrayList<>();
                            List<EmailMessageAttachment> emailMessageAttachmentList = new ArrayList<>();

                            for (MessageVo.MessagesDTO messagesDTO : messageList) {
                                if (lastMessageId.contains(messagesDTO.getId())) {
                                    continue;
                                }

                                MessageDetailVo messageDetail = EmailEngineUtil.getMessageDetail(email, messagesDTO.getId());

                                // 基本信息
                                EmailMessage emailMessage = new EmailMessage();
                                long id = IdWorker.getId();
                                emailMessage.setId(id);
                                emailMessage.setMessageId(messagesDTO.getId());
                                emailMessage.setEmailMailboxId(emailMailbox.getId());
                                emailMessage.setEmailMailboxPath(emailMailbox.getPath());
                                emailMessage.setUnseen(messagesDTO.getUnseen());
                                emailMessage.setFlagged(messagesDTO.getFlagged());
                                emailMessage.setSubject(messagesDTO.getSubject());
                                emailMessage.setFromDate(messagesDTO.getDate());

                                // 发件人
                                MessageVo.MessagesDTO.ToDTO from = messagesDTO.getFrom();
                                if (ObjectUtils.isNotEmpty(from)) {
                                    emailMessage.setFromName(from.getName());
                                    emailMessage.setFromAddress(from.getAddress());
                                }

                                // 正文内容
                                MessageDetailVo.TextDTO text = messageDetail.getText();
                                if (ObjectUtils.isNotEmpty(text)) {
                                    emailMessage.setHtmlContent(text.getHtml());
                                }

                                // 收件人
                                List<MessageVo.MessagesDTO.ToDTO> to = messagesDTO.getTo();
                                if (ObjectUtils.isNotEmpty(to)) {
                                    for (MessageVo.MessagesDTO.ToDTO toDTO : to) {
                                        EmailMessageSend emailMessageSend = new EmailMessageSend();
                                        emailMessageSend.setEmailMessageId(id);
                                        emailMessageSend.setMessageId(messagesDTO.getId());
                                        emailMessageSend.setType(SendConstants.TO);
                                        emailMessageSend.setName(toDTO.getName());
                                        emailMessageSend.setAddress(toDTO.getAddress());
                                        emailMessageSendList.add(emailMessageSend);
                                    }
                                }

                                // 抄送人
                                List<MessageVo.MessagesDTO.ToDTO> cc = messagesDTO.getCc();
                                if (ObjectUtils.isNotEmpty(cc)) {
                                    for (MessageVo.MessagesDTO.ToDTO toDTO : cc) {
                                        EmailMessageSend emailMessageSend = new EmailMessageSend();
                                        emailMessageSend.setEmailMessageId(id);
                                        emailMessageSend.setMessageId(messagesDTO.getId());
                                        emailMessageSend.setType(SendConstants.CC);
                                        emailMessageSend.setName(toDTO.getName());
                                        emailMessageSend.setAddress(toDTO.getAddress());
                                        emailMessageSendList.add(emailMessageSend);
                                    }
                                }

                                // 密送人
                                List<MessageVo.MessagesDTO.ToDTO> bcc = messagesDTO.getBcc();
                                if (ObjectUtils.isNotEmpty(bcc)) {
                                    for (MessageVo.MessagesDTO.ToDTO toDTO : cc) {
                                        EmailMessageSend emailMessageSend = new EmailMessageSend();
                                        emailMessageSend.setEmailMessageId(id);
                                        emailMessageSend.setMessageId(messagesDTO.getId());
                                        emailMessageSend.setType(SendConstants.BCC);
                                        emailMessageSend.setName(toDTO.getName());
                                        emailMessageSend.setAddress(toDTO.getAddress());
                                        emailMessageSendList.add(emailMessageSend);
                                    }
                                }

                                List<MessageVo.MessagesDTO.AttachmentsDTO> attachments = messagesDTO.getAttachments();
                                if (ObjectUtils.isNotEmpty(attachments)) {
                                    for (MessageVo.MessagesDTO.AttachmentsDTO attachment : attachments) {
                                        String attachmentId = attachment.getId();

                                        String newFileName = attachmentId + "." + FileUtil.getSuffix(attachment.getFilename());

                                        // 异常重试
                                        boolean execute = RetryUtil.execute(() -> EmailEngineUtil.downloadAttachment(email, attachmentId, newFileName));

                                        // TODO 下载失败加入日志
                                        if (execute) {
                                            EmailMessageAttachment emailMessageAttachment = new EmailMessageAttachment();
                                            emailMessageAttachment.setEmailMessageId(id);
                                            emailMessageAttachment.setMessageId(messagesDTO.getId());
                                            emailMessageAttachment.setName(attachment.getFilename());
                                            emailMessageAttachment.setPath(email + "\\" + newFileName);
                                            emailMessageAttachment.setAttachmentId(attachmentId);
                                            emailMessageAttachment.setSize(attachment.getEncodedSize());
                                            emailMessageAttachmentList.add(emailMessageAttachment);
                                        }

                                    }
                                }

                                emailMessageList.add(emailMessage);
                            }

                            emailMessageService.saveBatch(emailMessageList);
                            emailMessageSendService.saveBatch(emailMessageSendList);
                            emailMessageAttachmentService.saveBatch(emailMessageAttachmentList);

                            lastMessageId.clear();
                            for (EmailMessage emailMessage : emailMessageList) {
                                lastMessageId.add(emailMessage.getMessageId());
                            }

                        }
                );
            }
        });

    }

}
